@@ -10,21 +10,13 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclFileName.c,v 1.11 2000/02/01 11:49:24 hobbs Exp $
+ * RCS: @(#) $Id: tclFileName.c,v 1.13.2.3 2002/10/15 20:24:55 hobbs Exp $
  */
 
 #include "tclInt.h"
 #include "tclPort.h"
 #include "tclRegexp.h"
 
-/*
- * The following regular expression matches the root portion of a Windows
- * absolute or volume relative path.  It will match both UNC and drive relative
- * paths.
- */
-
-#define WIN_ROOT_PATTERN "^(([a-zA-Z]:)|[/\\\\][/\\\\]+([^/\\\\]+)[/\\\\]+([^/\\\\]+)|([/\\\\]))([/\\\\])*"
-
 /*
  * The following regular expression matches the root portion of a Macintosh
  * absolute path.  It will match degenerate Unix-style paths, tilde paths,
@@ -40,7 +32,6 @@
 
 typedef struct ThreadSpecificData {
     int initialized;
-    Tcl_Obj *winRootPatternPtr;
     Tcl_Obj *macRootPatternPtr;
 } ThreadSpecificData;
 
@@ -69,7 +60,7 @@ TclPlatformType tclPlatform = TCL_PLATFORM_UNIX;
 static char *		DoTildeSubst _ANSI_ARGS_((Tcl_Interp *interp,
 			    CONST char *user, Tcl_DString *resultPtr));
 static CONST char *	ExtractWinRoot _ANSI_ARGS_((CONST char *path,
-			    Tcl_DString *resultPtr, int offset));
+			    Tcl_DString *resultPtr, int offset, Tcl_PathType *typePtr));
 static void		FileNameCleanup _ANSI_ARGS_((ClientData clientData));
 static void		FileNameInit _ANSI_ARGS_((void));
 static int		SkipToChar _ANSI_ARGS_((char **stringPtr,
@@ -103,7 +94,6 @@ FileNameInit()
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
     if (!tsdPtr->initialized) {
 	tsdPtr->initialized = 1;
-	tsdPtr->winRootPatternPtr = Tcl_NewStringObj(WIN_ROOT_PATTERN, -1);
 	tsdPtr->macRootPatternPtr = Tcl_NewStringObj(MAC_ROOT_PATTERN, -1);
 	Tcl_CreateThreadExitHandler(FileNameCleanup, NULL);
     }
@@ -131,7 +121,6 @@ FileNameCleanup(clientData)
     ClientData clientData;	/* Not used. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-    Tcl_DecrRefCount(tsdPtr->winRootPatternPtr);
     Tcl_DecrRefCount(tsdPtr->macRootPatternPtr);
     tsdPtr->initialized = 0;
 }
@@ -157,58 +146,95 @@ FileNameCleanup(clientData)
  */
 
 static CONST char *
-ExtractWinRoot(path, resultPtr, offset)
+ExtractWinRoot(path, resultPtr, offset, typePtr)
     CONST char *path;		/* Path to parse. */
     Tcl_DString *resultPtr;	/* Buffer to hold result. */
     int offset;			/* Offset in buffer where result should be
 				 * stored. */
+    Tcl_PathType *typePtr;	/* Where to store pathType result */
 {
-    int length;
-    Tcl_RegExp re;
-    char *dummy, *tail, *drive, *hostStart, *hostEnd, *shareStart,
-	*shareEnd, *lastSlash;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    if (path[0] == '/' || path[0] == '\\') {
+	/* Might be a UNC or Vol-Relative path */
+	char *host, *share, *tail;
+	int hlen, slen;
+	if (path[1] != '/' && path[1] != '\\') {
+	    Tcl_DStringSetLength(resultPtr, offset);
+	    *typePtr = TCL_PATH_VOLUME_RELATIVE;
+	    Tcl_DStringAppend(resultPtr, "/", 1);
+	    return &path[1];
+	}
+	host = (char *)&path[2];
 
-    /*
-     * Initialize the path name parser for Windows path names.
-     */
+	/* Skip separators */
+	while (host[0] == '/' || host[0] == '\\') host++;
 
-    FileNameInit();
+	for (hlen = 0; host[hlen];hlen++) {
+	    if (host[hlen] == '/' || host[hlen] == '\\')
+		break;
+	}
+	if (host[hlen] == 0 || host[hlen+1] == 0) {
+	    /* 
+	     * The path given is simply of the form 
+	     * '/foo', '//foo', '/////foo' or the same
+	     * with backslashes.  If there is exactly
+	     * one leading '/' the path is volume relative
+	     * (see filename man page).  If there are more
+	     * than one, we are simply assuming they
+	     * are superfluous and we trim them away.
+	     * (An alternative interpretation would
+	     * be that it is a host name, but we have
+	     * been documented that that is not the case).
+	     */
+	    *typePtr = TCL_PATH_VOLUME_RELATIVE;
+	    Tcl_DStringAppend(resultPtr, "/", 1);
+	    return &path[2];
+	}
+	Tcl_DStringSetLength(resultPtr, offset);
+	share = &host[hlen];
 
-    re = Tcl_GetRegExpFromObj(NULL, tsdPtr->winRootPatternPtr, REG_ADVANCED);
+	/* Skip separators */
+	while (share[0] == '/' || share[0] == '\\') share++;
 
-    /*
-     * Match the root portion of a Windows path name.
-     */
+	for (slen = 0; share[slen];slen++) {
+	    if (share[slen] == '/' || share[slen] == '\\')
+		break;
+	}
+	Tcl_DStringAppend(resultPtr, "//", 2);
+	Tcl_DStringAppend(resultPtr, host, hlen);
+	Tcl_DStringAppend(resultPtr, "/", 1);
+	Tcl_DStringAppend(resultPtr, share, slen);
 
-    if (!Tcl_RegExpExec(NULL, re, path, path)) {
+	tail = &share[slen];
+
+	/* Skip separators */
+	while (tail[0] == '/' || tail[0] == '\\') tail++;
+
+	*typePtr = TCL_PATH_ABSOLUTE;
+	return tail;
+    } else if (*path && path[1] == ':') {
+	/* Might be a drive sep */
+	Tcl_DStringSetLength(resultPtr, offset);
+
+	if (path[2] != '/' && path[2] != '\\') {
+	    *typePtr = TCL_PATH_VOLUME_RELATIVE;
+	    Tcl_DStringAppend(resultPtr, path, 2);
+	    return &path[2];
+	} else {
+	    char *tail = (char*)&path[3];
+
+	    /* Skip separators */
+	    while (*tail && (tail[0] == '/' || tail[0] == '\\')) tail++;
+
+	    *typePtr = TCL_PATH_ABSOLUTE;
+	    Tcl_DStringAppend(resultPtr, path, 2);
+	    Tcl_DStringAppend(resultPtr, "/", 1);
+
+	    return tail;
+	}
+    } else {
+	*typePtr = TCL_PATH_RELATIVE;
 	return path;
     }
-
-    Tcl_DStringSetLength(resultPtr, offset);
-
-    Tcl_RegExpRange(re, 0, &dummy, &tail);
-    Tcl_RegExpRange(re, 2, &drive, &dummy);
-    Tcl_RegExpRange(re, 3, &hostStart, &hostEnd);
-    Tcl_RegExpRange(re, 4, &shareStart, &shareEnd);
-    Tcl_RegExpRange(re, 6, &lastSlash, &dummy);
-
-    if (drive != NULL) {
-	Tcl_DStringAppend(resultPtr, drive, 2);
-	if (lastSlash != NULL) {
-	    Tcl_DStringAppend(resultPtr, "/", 1);
-	}
-    } else if (shareStart != NULL) {
-	Tcl_DStringAppend(resultPtr, "//", 2);
-	length = hostEnd - hostStart;
-	Tcl_DStringAppend(resultPtr, hostStart, length);
-	Tcl_DStringAppend(resultPtr, "/", 1);
-	length = shareEnd - shareStart;
-	Tcl_DStringAppend(resultPtr, shareStart, length);
-    } else {
-	Tcl_DStringAppend(resultPtr, "/", 1);
-    }
-    return tail;
 }
 
 /*
@@ -278,30 +304,11 @@ Tcl_GetPathType(path)
 	
 	case TCL_PLATFORM_WINDOWS:
 	    if (path[0] != '~') {
-		tsdPtr = TCL_TSD_INIT(&dataKey);
+		Tcl_DString ds;
 
-		/*
-		 * Since we have eliminated the easy cases, check for
-		 * drive relative paths using the regular expression.
-		 */
-
-		FileNameInit();
-		re = Tcl_GetRegExpFromObj(NULL, tsdPtr->winRootPatternPtr,
-			REG_ADVANCED);
-
-		if (Tcl_RegExpExec(NULL, re, path, path)) {
-		    char *drive, *dummy, *unixRoot, *lastSlash;
-
-		    Tcl_RegExpRange(re, 2, &drive, &dummy);
-		    Tcl_RegExpRange(re, 5, &unixRoot, &dummy);
-		    Tcl_RegExpRange(re, 6, &lastSlash, &dummy);
-		    
-		    if (unixRoot || (drive && !lastSlash)) {
-			type = TCL_PATH_VOLUME_RELATIVE;
-		    }
-		} else {
-		    type = TCL_PATH_RELATIVE;
-		}
+		Tcl_DStringInit(&ds);
+		(VOID)ExtractWinRoot(path, &ds, 0, &type);
+		Tcl_DStringFree(&ds);
 	    }
 	    break;
     }
@@ -440,6 +447,19 @@ SplitUnixPath(path, bufPtr)
      * Deal with the root directory as a special case.
      */
 
+#ifdef __QNX__
+    /*
+     * Check for QNX //<node id> prefix
+     */
+    if ((path[0] == '/') && (path[1] == '/')
+	    && isdigit(UCHAR(path[2]))) { /* INTL: digit */
+	path += 3;
+	while (isdigit(UCHAR(*path))) { /* INTL: digit */
+	    ++path;
+	}
+    }
+#endif
+
     if (path[0] == '/') {
 	Tcl_DStringAppend(bufPtr, "/", 2);
 	p = path+1;
@@ -497,8 +517,9 @@ SplitWinPath(path, bufPtr)
 {
     int length;
     CONST char *p, *elementStart;
+    Tcl_PathType type = TCL_PATH_ABSOLUTE;
 
-    p = ExtractWinRoot(path, bufPtr, 0);
+    p = ExtractWinRoot(path, bufPtr, 0, &type);
 
     /*
      * Terminate the root portion, if we matched something.
@@ -747,6 +768,7 @@ Tcl_JoinPath(argc, argv, resultPtr)
     Tcl_DString buffer;
     char c, *dest;
     CONST char *p;
+    Tcl_PathType type = TCL_PATH_ABSOLUTE;
 
     Tcl_DStringInit(&buffer);
     oldLength = Tcl_DStringLength(resultPtr);
@@ -835,7 +857,7 @@ Tcl_JoinPath(argc, argv, resultPtr)
 	     */
 
 	    for (i = 0; i < argc; i++) {
-		p = ExtractWinRoot(argv[i], resultPtr, oldLength);
+		p = ExtractWinRoot(argv[i], resultPtr, oldLength, &type);
 		length = Tcl_DStringLength(resultPtr);
 		
 		/*
@@ -988,8 +1010,6 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
     Tcl_DString *bufferPtr;	/* Uninitialized or free DString filled
 				 * with name after tilde substitution. */
 {
-    register char *p;
-
     /*
      * Handle tilde substitutions, if needed.
      */
@@ -999,7 +1019,7 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 	char **argv;
 	Tcl_DString temp;
 
-	Tcl_SplitPath(name, &argc, &argv);
+	Tcl_SplitPath(name, &argc, (char ***) &argv);
 	
 	/*
 	 * Strip the trailing ':' off of a Mac path before passing the user
@@ -1019,12 +1039,12 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 	    return NULL;
 	}
 	Tcl_DStringInit(bufferPtr);
-	Tcl_JoinPath(argc, argv, bufferPtr);
+	Tcl_JoinPath(argc, (char **) argv, bufferPtr);
 	Tcl_DStringFree(&temp);
 	ckfree((char*)argv);
     } else {
 	Tcl_DStringInit(bufferPtr);
-	Tcl_JoinPath(1, &name, bufferPtr);
+	Tcl_JoinPath(1, (char **) &name, bufferPtr);
     }
 
     /*
@@ -1033,11 +1053,30 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
      */
 
     if (tclPlatform == TCL_PLATFORM_WINDOWS) {
+#if defined(__CYGWIN__) && defined(__WIN32__)
+	extern int cygwin_conv_to_win32_path 
+	    _ANSI_ARGS_((CONST char *, char *));
+	char winbuf[MAX_PATH];
+
+	/*
+	 * In the Cygwin world, call conv_to_win32_path in order to use the
+	 * mount table to translate the file name into something Windows will
+	 * understand.  Take care when converting empty strings!
+	 */
+	if (Tcl_DStringLength(bufferPtr)) {
+	    cygwin_conv_to_win32_path(Tcl_DStringValue(bufferPtr), winbuf);
+	    Tcl_DStringFree(bufferPtr);
+	    Tcl_DStringAppend(bufferPtr, winbuf, -1);
+	}
+#else /* __CYGWIN__ && __WIN32__ */
+
+	register char *p;
 	for (p = Tcl_DStringValue(bufferPtr); *p != '\0'; p++) {
 	    if (*p == '/') {
 		*p = '\\';
 	    }
 	}
+#endif /* __CYGWIN__ && __WIN32__ */
     }
     return Tcl_DStringValue(bufferPtr);
 }
@@ -1887,6 +1926,25 @@ TclDoGlob(interp, separators, headPtr, tail, types)
 	     * element.  Add an extra slash if this is a UNC path.
 	     */
 
+#if defined(__CYGWIN__) && defined(__WIN32__)
+	    {
+
+	    extern int cygwin_conv_to_win32_path 
+	    	_ANSI_ARGS_((CONST char *, char *));
+	    char winbuf[MAX_PATH];
+
+	    /*
+	     * In the Cygwin world, call conv_to_win32_path in order to use
+	     * the mount table to translate the file name into something
+	     * Windows will understand.
+	     */
+	    cygwin_conv_to_win32_path(Tcl_DStringValue(headPtr), winbuf);
+	    Tcl_DStringFree(headPtr);
+	    Tcl_DStringAppend(headPtr, winbuf, -1);
+
+	    }
+#endif /* __CYGWIN__ && __WIN32__ */
+
 	    if (*name == ':') {
 		Tcl_DStringAppend(headPtr, ":", 1);
 		if (count > 1) {
