@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUtf.c,v 1.10 1999/07/22 01:08:05 redman Exp $
+ * RCS: @(#) $Id: tclUtf.c,v 1.11.2.2 2001/09/13 19:33:19 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -111,7 +111,7 @@ static int UtfCount _ANSI_ARGS_((int ch));
  *---------------------------------------------------------------------------
  */
  
-static int
+INLINE static int
 UtfCount(ch)
     int ch;			/* The Tcl_UniChar whose size is returned. */
 {
@@ -672,9 +672,6 @@ Tcl_UtfPrev(str, start)
 	    break;
 	} 
 	if (byte >= 0xC0) {
-	    if (totalBytes[byte] != i + 1) {
-		break;
-	    }
 	    return (char *) look;
 	}
 	look--;
@@ -781,7 +778,8 @@ Tcl_UtfBackslash(src, readPtr, dst)
 				 * backslash sequence. */
 {
     register CONST char *p = src+1;
-    int result, count, n;
+    Tcl_UniChar result;
+    int count, n;
     char buf[TCL_UTF_MAX];
 
     if (dst == NULL) {
@@ -864,31 +862,44 @@ Tcl_UtfBackslash(src, readPtr, dst)
             count = 1;
             break;
 	default:
-	    if (isdigit(UCHAR(*p))) { /* INTL: digit */
+	    /*
+	     * Check for an octal number \oo?o?
+	     */
+	    if (isdigit(UCHAR(*p)) && (UCHAR(*p) < '8')) { /* INTL: digit */
 		result = (unsigned char)(*p - '0');
 		p++;
-		if (!isdigit(UCHAR(*p))) { /* INTL: digit */
+		if (!isdigit(UCHAR(*p)) || (UCHAR(*p) >= '8')) { /* INTL: digit */
 		    break;
 		}
 		count = 3;
 		result = (unsigned char)((result << 3) + (*p - '0'));
 		p++;
-		if (!isdigit(UCHAR(*p))) { /* INTL: digit */
+		if (!isdigit(UCHAR(*p)) || (UCHAR(*p) >= '8')) { /* INTL: digit */
 		    break;
 		}
 		count = 4;
 		result = (unsigned char)((result << 3) + (*p - '0'));
 		break;
 	    }
+	    if (UCHAR(*p) < UNICODE_SELF) {
 	    result = *p;
 	    count = 2;
+	    } else {
+		/*
+		 * We have to convert here because the user has put a
+		 * backslash in front of a multi-byte utf-8 character.
+		 * While this means nothing special, we shouldn't break up
+		 * a correct utf-8 character. [Bug #217987] test subst-3.2
+		 */
+		count = Tcl_UtfToUniChar(p, &result) + 1; /* +1 for '\' */
+	    }
 	    break;
     }
 
     if (readPtr != NULL) {
 	*readPtr = count;
     }
-    return Tcl_UniCharToUtf(result, dst);
+    return Tcl_UniCharToUtf((int) result, dst);
 }
 
 /*
