@@ -9,12 +9,11 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinPipe.c,v 1.11.2.2 2001/04/03 22:54:39 hobbs Exp $
+ * RCS: @(#) $Id: tclWinPipe.c,v 1.11.2.5 2002/10/15 19:38:48 hobbs Exp $
  */
 
 #include "tclWinInt.h"
 
-#include <dos.h>
 #include <fcntl.h>
 #include <io.h>
 #include <sys/stat.h>
@@ -1522,10 +1521,10 @@ BuildCommandLine(
 	}
 
 	quote = 0;
-	if (argv[i][0] == '\0') {
+	if (arg[0] == '\0') {
 	    quote = 1;
 	} else {
-	    for (start = argv[i]; *start != '\0'; start++) {
+	    for (start = arg; *start != '\0'; start++) {
 		if (isspace(*start)) { /* INTL: ISO space. */
 		    quote = 1;
 		    break;
@@ -1824,14 +1823,6 @@ PipeClose2Proc(
 	 */
 
 	if (pipePtr->readThread) {
-	    /*
-	     * Note that we need to guard against terminating the thread while
-	     * it is in the middle of Tcl_ThreadAlert because it won't be able
-	     * to release the notifier lock.
-	     */
-
-	    Tcl_MutexLock(&pipeMutex);
-
 	    /*
 	     * The thread may already have closed on it's own.  Check it's
 	     * exit code.
@@ -1857,24 +1848,31 @@ PipeClose2Proc(
 
 		if (exitCode == STILL_ACTIVE) {
 		    /*
-		     * The thread must be blocked waiting for the pipe to become
-		     * readable in ReadFile().  There isn't a clean way to exit
-		     * the thread from this condition.  We should terminate the
-		     * child process instead to get the reader thread to fall out of
-		     * ReadFile with a FALSE.  (below) is not the correct way to do
-		     * this, but will stay here until a better solution is found.
+		     * The thread must be blocked waiting for the pipe to
+		     * become readable in ReadFile().  There isn't a clean way
+		     * to exit the thread from this condition.  We should
+		     * terminate the child process instead to get the reader
+		     * thread to fall out of ReadFile with a FALSE.  (below) is
+		     * not the correct way to do this, but will stay here until
+		     * a better solution is found.
+		     *
+		     * Note that we need to guard against terminating the
+		     * thread while it is in the middle of Tcl_ThreadAlert
+		     * because it won't be able to release the notifier lock.
 		     */
 
+		    Tcl_MutexLock(&pipeMutex);
+
 		    /* BUG: this leaks memory */
 		    TerminateThread(pipePtr->readThread, 0);
 
 		    /* Wait for the thread to terminate. */
 		    WaitForSingleObject(pipePtr->readThread, INFINITE);
+
+		    Tcl_MutexUnlock(&pipeMutex);
 		}
 	    }
 
-	    Tcl_MutexUnlock(&pipeMutex);
-
 	    CloseHandle(pipePtr->readThread);
 	    CloseHandle(pipePtr->readable);
 	    CloseHandle(pipePtr->startReader);
